#ifndef _CPUINFO_HH_
#define _CPUINFO_HH_

#include <string>
#include <map>
#include <fstream>
#include <devscan.hh>
#include <hwdb.hh>

namespace libhw
{
#define MSR_IA32_PLATFORM_ID	0x17
#define MSR_IA32_UCODE_REV		0x8b

#define MSR_IA32_PERF_STATUS		0x198
#define MSR_IA32_PERF_CTL		0x199
#define MSR_IA32_THERM_CONTROL		0x19a
#define MSR_IA32_THERM_STATUS		0x19c
#define MSR_PM_THERM2_CTL		0x19d
#define MSR_IA32_MISC_ENABLE		0x1a0

  float cpu_estimate_MHz(int cpu);
  void cpuid (int, unsigned int, unsigned int *, unsigned int *, unsigned int *, unsigned int *);
  void dump_cpuid(const std::string & vendor, int cpu);
  int read_msr(int cpu, unsigned int idx, unsigned long long *val);

  class cpuinfo
    {
    hwdb cpudb;
    hwdb amd_fid;
    hwdb amd_vid;
    hwdb &hwlist_;
    public:
      cpuinfo(libhw::hwdb & hwlist);
static    std::string vend_id;
      int count();
      int operator() (std::string path, pot::attrs_t node);
    private:
      void cpuid_amd();
      void cpuid_intel();
      void parse_x86(std::string & id,std::string & value);
      std::map < int, cpuinfo > cpus;
      int lastcpuid;
    };
}
#endif
