#ifndef _PCI_HH_
#define _PCI_HH_

#include <hwdb.hh>
#include <hwlist.hh>
#include <devscan.hh>
extern "C" {
#include <pci/pci.h>
}

typedef u8 byte;

/*
 * pci module info
 */
typedef struct {
  std::string module;
  unsigned vendor;
  unsigned device;
  unsigned subvendor;
  unsigned subdevice;
  unsigned pciclass;
  unsigned classmask;
} hwdb_pci_t;



namespace libhw
{

  class pci_db {
  public:
    pci_db() {};
    void pci_db_init(std::string k)
    {
      kernel = k; 
      parse_pcimap();
      parse_modulesdep();
    };
    std::set <std::string> pci_get_drivers(unsigned device, unsigned vendor, unsigned subdevice, unsigned subvendor, unsigned pciclass);
    std::string file2modname(char *filename);
    bool checkdriver(std::string driver);
  private:
    void parse_pcimap();
    void parse_modulesdep();
    std::vector <hwdb_pci_t> pci;
    std::set <std::string> modules;
    std::string kernel;
  };

  struct device {
    struct device *next;
    struct pci_dev *dev;
    unsigned int config_cached, config_bufsize;
    byte *config;				/* Cached configuration space data */
    byte *present;			/* Maps which configuration bytes are present */
  };

  class pci
  {
    hwlist& hwlist_;
    hwdb pcidb;
    hwdb classdb;
  public:
    pci(hwlist & hwlist);
    int show();
    int copy(std::string & name, pot::base_node<const pot>& node);
    int operator() (std::string path, pot::attrs_t node);
    pci_db pcikerndb;
    bool checkdriver(std::string driver) { return pcikerndb.checkdriver(driver);};
  private:
    struct device * scan_device(struct pci_dev *p);
    int  config_fetch(struct device *d, unsigned int pos, unsigned int len);
    void pciclass_set_info( const std::string & device, const unsigned short pciclass, const unsigned short progif);
    bool check_conf_range(struct device *d, unsigned int pos, unsigned int len);
    u_int32_t get_conf_long(struct device * d, unsigned int pos);
    u_int16_t get_conf_word(struct device * d, unsigned int pos);
    u_int8_t get_conf_byte(struct device * d, unsigned int pos);
  };
}
  
#endif
